/*
+----------------------------------------------------------------------+
|                    Class: CStarUtils                                 |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
+----------------------------------------------------------------------+
*/
package com.xqual.xtrigger.star;

import com.xqual.xcommon.utils.CTextUtils;
import com.xqual.xtrigger.ITrigger;

/**
 * The <code>CStarUtils</code> class defines utilities for the STAR trigger implementation
 * @author egavaldo
 */
public class CStarUtils {

    // +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+
	
	// when storing data in database, all ' must be replaced by ''
 	public static String escapeStringForDatabase(String input) {
  		if (input == null) return ""; // in Oracle, '' is considered as NULL; 00A0 is a NO-BREAK SPACE character

  		String output = input;
		output = output.replaceAll("'", "''");
		//if (CStudioGlobals.databaseType != IConstantsSql.DATABASE_MSSQL) {
			output = output.replaceAll("\\\\", "\\\\\\\\"); // if we have a text including \t it does not mean tab
		//}
		return CTextUtils.removeInvisibleCharacters(output);
	}
 	
 	public static boolean existsInArray(String input, String[] array) {
 		for (int i=0; i<array.length; i++) {
 			if (array[i].equals(input))
 				return true;
 		}
 		return false;
 	}
 	
	public static String replaceTabLfAndCrWithSpace(String input) {
  		if (input == null) return "";

		// forbidden invisible characters (copy/paste from Word etc.): we just delete them
  		String output = input;
 		output = output.replaceAll("\\x0a", "\\\\n"); // LF
 		output = output.replaceAll("\\x0d", "\\\\n"); // CR
		return output;		
	}
	
    public static String statusToString(short status) {
    	switch (status) {
    	case ITrigger.INTEGRATED_BTD_STATUS_NEW:
    		return "New";
    	case ITrigger.INTEGRATED_BTD_STATUS_ASSIGNED:
    		return "Assigned";
    	case ITrigger.INTEGRATED_BTD_STATUS_ACKNOWLEDGED:
    		return "Acknowledged";
    	case ITrigger.INTEGRATED_BTD_STATUS_RESOLVED:
    		return "Resolved";
    	case ITrigger.INTEGRATED_BTD_STATUS_CLOSED:
    		return "Closed";
    	default:
    		return "???";
    	}
    }
    
    public static String severityToString(short severity) {
    	switch (severity) {
    	case ITrigger.INTEGRATED_BTD_SEVERITY_BLOCKING:
    		return "Blocking";
    	case ITrigger.INTEGRATED_BTD_SEVERITY_MAJOR:
    		return "Major";
    	case ITrigger.INTEGRATED_BTD_SEVERITY_MINOR:
    		return "Minor";
    	case ITrigger.INTEGRATED_BTD_SEVERITY_COSMETIC:
    		return "Cosmetic";
    	case ITrigger.INTEGRATED_BTD_SEVERITY_ENHANCEMENT:
    		return "Enhancement";
    	default:
    		return "???";
    	}
    }
    
    public static String priorityToString(short priority) {
    	switch (priority) {
    	case ITrigger.INTEGRATED_BTD_PRIORITY_HIGH:
    		return "High";
    	case ITrigger.INTEGRATED_BTD_PRIORITY_NORMAL:
    		return "Normal";
    	case ITrigger.INTEGRATED_BTD_PRIORITY_LOW:
    		return "Low";
    	default:
    		return "???";
    	}
    }

}
