/*
+----------------------------------------------------------------------+
|                    Class: CTrigger                                   |
|                                                                      |
| Developer:   Eric Gavaldo (XQual)                                    |
| Launcher:	   STAR                                                    |
| Version:     1.2                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xtrigger.star;

import java.awt.HeadlessException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.json.JSONArray;
import org.json.JSONObject;

import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CTextUtils;
import com.xqual.xtrigger.CDatabaseConnectionProperties;
import com.xqual.xtrigger.CPropertyChange;
import com.xqual.xtrigger.CTrigger;
import com.xqual.xtrigger.CTriggerSqlConnector;
import com.xqual.xtrigger.CTriggerStatus;
import com.xqual.xtrigger.star.rest.CStarRestEngine;
import com.xqual.xtrigger.star.sql.CStarSqlCreateEngine;
import com.xqual.xtrigger.star.sql.CStarSqlDeleteEngine;
import com.xqual.xtrigger.star.sql.CStarSqlGetEngine;
import com.xqual.xtrigger.star.sql.CStarSqlUpdateEngine;



/**
 * The <code>CTriggerImpl</code> implementation of <code>ITrigger</code> for STAR.
 * @author rball
 */
public class CTriggerImpl extends CTrigger implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{STAR        }  ";
	
	// Connection to XStudio Database
	short connectionStatus = -1;
	
	Vector<String> filterActivity;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+
	public CTriggerImpl(CDatabaseConnectionProperties databaseConnectionProperties) {
		super(TRACE_HEADER);
		
		System.out.println("- - ->> jnlpMode                                    = " + databaseConnectionProperties.jnlpMode);
		System.out.println("- - ->> jnlpCodebaseURL                             = " + databaseConnectionProperties.jnlpCodebaseURL);
		System.out.println("- - ->> jarPath                                     = " + databaseConnectionProperties.jarPath);
		System.out.println("- - ->> classPath                                   = " + databaseConnectionProperties.classPath);
		System.out.println("- - ->> template                                    = " + databaseConnectionProperties.template);
		System.out.println("- - ->> server                                      = " + databaseConnectionProperties.server);
		System.out.println("- - ->> name                                        = " + databaseConnectionProperties.name);
		System.out.println("- - ->> databaseType                                = " + databaseConnectionProperties.databaseType);
		System.out.println("- - ->> username                                    = " + databaseConnectionProperties.username);
		System.out.println("- - ->> password                                    = " + databaseConnectionProperties.password);
		System.out.println("- - ->> maxNumberConnections                        = " + databaseConnectionProperties.maxNumberConnections);
		System.out.println("- - ->> sshTunnelingEnabled                         = " + databaseConnectionProperties.sshTunnelingEnabled);
		System.out.println("- - ->> sshTunnelingServer                          = " + databaseConnectionProperties.sshTunnelingServer);
		System.out.println("- - ->> sshTunnelingPort                            = " + databaseConnectionProperties.sshTunnelingPort);
		System.out.println("- - ->> sshTunnelingUseXStudioLoginUsernamePassword = " + databaseConnectionProperties.sshTunnelingUseXStudioLoginUsernamePassword);
		System.out.println("- - ->> sshTunnelingUsername                        = " + databaseConnectionProperties.sshTunnelingUsername);
		System.out.println("- - ->> sshTunnelingPassword                        = " + databaseConnectionProperties.sshTunnelingPassword);
		System.out.println("- - ->> sshTunnelingSourcePort                      = " + databaseConnectionProperties.sshTunnelingSourcePort);
		System.out.println("- - ->> sshTunnelingDestinationPort                 = " + databaseConnectionProperties.sshTunnelingDestinationPort);
		
		System.out.println("setting up activity filter...");
		filterActivity = new Vector<String>();
		filterActivity.add("SSS");
		
		System.out.println("setting up SQL engine...");
		connectionStatus = CTriggerSqlConnector.connectToXStudioDB(databaseConnectionProperties.jnlpMode, databaseConnectionProperties.jnlpCodebaseURL,
																   databaseConnectionProperties.jarPath, databaseConnectionProperties.classPath,
																
																   databaseConnectionProperties.template, databaseConnectionProperties.server, databaseConnectionProperties.name, databaseConnectionProperties.databaseType,
																   databaseConnectionProperties.username, 
																   databaseConnectionProperties.password,
																
																   databaseConnectionProperties.maxNumberConnections,
																
																   databaseConnectionProperties.sshTunnelingEnabled,
																   databaseConnectionProperties.sshTunnelingServer,
																   databaseConnectionProperties.sshTunnelingPort,
																   databaseConnectionProperties.sshTunnelingUseXStudioLoginUsernamePassword,
																   databaseConnectionProperties.sshTunnelingUsername,
																   databaseConnectionProperties.sshTunnelingPassword,
																   databaseConnectionProperties.sshTunnelingSourcePort,
																   databaseConnectionProperties.sshTunnelingDestinationPort);
	}

	// +------------------------------------+
	// | Initialize                         |
	// +------------------------------------+

	@Override
	public CTriggerStatus initialize() {
		System.out.println(TRACE_HEADER + "initialize()");
		
		System.out.println("****** connectionStatus = " + connectionStatus);
		if (connectionStatus != CTriggerSqlConnector.CONNECTION_SUCCESS) {
			return new CTriggerStatus(RESULT_FAILURE, "Couldn't connect to the XStudio database!");
		}
		
		
		// set the sql engines
		System.out.println("setting xstudio sql engines...");
		CStarGlobals.getEngine = new CStarSqlGetEngine();
		if (CStarGlobals.getEngine == null) {
			return new CTriggerStatus(RESULT_FAILURE, "Couldn't create XStudio SQL get engine!");
		}
		CStarGlobals.updateEngine = new CStarSqlUpdateEngine();
		if (CStarGlobals.updateEngine == null) {
			return new CTriggerStatus(RESULT_FAILURE, "Couldn't create XStudio SQL update engine!");
		}
		CStarGlobals.createEngine = new CStarSqlCreateEngine();
		if (CStarGlobals.createEngine == null) {
			return new CTriggerStatus(RESULT_FAILURE, "Couldn't create XStudio SQL create engine!");
		}
		CStarGlobals.deleteEngine = new CStarSqlDeleteEngine();
		if (CStarGlobals.deleteEngine == null) {
			return new CTriggerStatus(RESULT_FAILURE, "Couldn't create XStudio SQL delete engine!");
		}

		
		// set the sql engines
		System.out.println("setting STAR REST engines...");
		CStarGlobals.starRestEngine = new CStarRestEngine();
		if (CStarGlobals.starRestEngine == null) {
			return new CTriggerStatus(RESULT_FAILURE, "Couldn't create STAR REST engine!");
		}

		return new CTriggerStatus();
	}

	// +------------------------------------+
	// | Requirement events                 |
	// +------------------------------------+

	@Override
	public CTriggerStatus onRequirementsTreeRefreshed() {
		System.out.println(TRACE_HEADER + "initialize()");
		return new CTriggerStatus();
	}

	@Override
	public CTriggerStatus onRequirementCreated(int id, String name, String description, short status, short priority, int risk) {
		System.out.println(TRACE_HEADER + "onRequirementCreated()");
		return new CTriggerStatus();
	}

	@Override
	public CTriggerStatus onRequirementUpdated(int id, Vector<CPropertyChange> changeList) {
		System.out.println(TRACE_HEADER + "onRequirementUpdated()");
		return new CTriggerStatus();
	}
	
	@Override
	public CTriggerStatus onRequirementsDeleted(Vector<Integer> ids) {
		System.out.println(TRACE_HEADER + "onRequirementsDeleted()");
		return new CTriggerStatus();
	}

	@Override
	public CTriggerStatus onRequirementsMoved(Vector<Integer> ids, int folderId) {
		System.out.println(TRACE_HEADER + "onRequirementsMoved()");
		return new CTriggerStatus();
	}

	// +------------------------------------+
	// | Test events                        |
	// +------------------------------------+

	@Override
	public CTriggerStatus onTestsTreeRefreshed() {
		System.out.println(TRACE_HEADER + "onTestsTreeRefreshed()");
		return new CTriggerStatus();
	}
	
	@Override
	public CTriggerStatus onTestCreated(int id, String name, short type, short status, short priority) {
		System.out.println(TRACE_HEADER + "onTestCreated()");
		return new CTriggerStatus();
	}

	@Override
	public CTriggerStatus onTestUpdated(int id, Vector<CPropertyChange> changeList) {
		System.out.println(TRACE_HEADER + "onTestUpdated()");
		return new CTriggerStatus();
	}
	
	@Override
	public CTriggerStatus onTestsDeleted(Vector<Integer> ids) {
		System.out.println(TRACE_HEADER + "onTestsDeleted()");
		return new CTriggerStatus();
	}

	@Override
	public CTriggerStatus onTestsMoved(Vector<Integer> ids, int folderId) {
		System.out.println(TRACE_HEADER + "onTestsMoved()");
		return new CTriggerStatus();
	}

	@Override
	public CTriggerStatus onTestcaseCreated(int id, String name, String description, boolean readyForManualRun, boolean readyForAutomatedRun, String procedure) {
		System.out.println(TRACE_HEADER + "onTestcaseCreated()");
		return new CTriggerStatus();
	}

	@Override
	public CTriggerStatus onTestcaseUpdated(int id, Vector<CPropertyChange> changeList) {
		System.out.println(TRACE_HEADER + "onTestcaseUpdated()");
		return new CTriggerStatus();
	}

	@Override
	public CTriggerStatus onTestcasesDeleted(Vector<Integer> ids) {
		System.out.println(TRACE_HEADER + "onTestcasesDeleted()");
		return new CTriggerStatus();
	}

	@Override
	public CTriggerStatus onTestcaseExecuted(int id, short status) {
		System.out.println(TRACE_HEADER + "onTestcaseExecuted()");
		return new CTriggerStatus();
	}

	// +------------------------------------+
	// | Defect events                      |
	// +------------------------------------+

	@Override
	public CTriggerStatus onBugsTreeRefreshed() {
		System.out.println(TRACE_HEADER + "---------------------------------------------------------");
		System.out.println(TRACE_HEADER + "onBugsTreeRefreshed()");
		System.out.println(TRACE_HEADER + "---------------------------------------------------------");
		// Fetch all tasks properties from TMS system
		// Compare with current values in XStudio's database and update only if necessary
		// WARNING: This can be very slow!
		String jsonContent = CStarGlobals.starRestEngine.qryTasks(IStarConstants.REFRESH_START_DATE);
		
		JSONObject rootObject = new JSONObject(jsonContent);
		boolean success  = rootObject.getBoolean("Success");
		String message   = rootObject.getString("Msg");

		System.out.println(TRACE_HEADER + "success: " + success);
		System.out.println(TRACE_HEADER + "message: " + message);
		if (!success) {
			try {
				JOptionPane.showMessageDialog(null, "Error returned by TMS: \"" + message + "\"\nTask couldn't be fetched from TMS!");
			} catch (HeadlessException e) {
			}
			
			return new CTriggerStatus(RESULT_FAILURE, message);
		}
		
    	JSONArray jsonArray = rootObject.getJSONArray("Data");
        if (jsonArray != null) {
        	
        	// retrieve the tasks with their id and status
        	HashMap<String, String> hashMapTaskIdStatus = new HashMap<String, String>();
        	System.out.println(TRACE_HEADER + "there are " + jsonArray.length() + " tasks to retrieve");
        	for (int k=0; k<jsonArray.length(); k++) {
        		JSONObject taskObject = (JSONObject) jsonArray.get(k);
        		
        		String taskId        = taskObject.getString("TaskID");   // i.e. "03271"
        		String taskActivity  = taskObject.getString("Activity"); // i.e. "SSS"
        		String taskStatus    = taskObject.getString("Status");   // i.e. "GI"
        		//Object taskCLSysId = taskObject.get("CLSysID");        // i.e. "123" (defectId)
				
        		if (filterActivity.contains(taskActivity)) {
        			//System.out.println(TRACE_HEADER + "task: " + taskId + " / " + taskActivity + " / " + taskStatus + " / " + taskCLSysId);
        			hashMapTaskIdStatus.put(taskId, taskStatus);
        		} else {
        			//System.err.println(TRACE_HEADER + "FILTERED task: " + taskId + " / " + taskActivity + " / " + taskStatus + " / " + taskCLSysId);
        		}
        	}
        	
			System.out.println(TRACE_HEADER + "hashMapTaskIdStatus: " + hashMapTaskIdStatus);
			
			// retrieve the defects with their external_id and status to check if there are some update to apply
			HashMap<String, String> hashMapIntegratedDefectIdExternalId = CStarGlobals.getEngine.getIntegratedDefectIdExternalId();
			System.out.println(TRACE_HEADER + "hashMapIntegratedDefectIdExternalId: " + hashMapIntegratedDefectIdExternalId);
			
			HashMap<String, String> hashMapIntegratedDefectExternalIdStatus = CStarGlobals.getEngine.getIntegratedDefectExternalIdStatus();
			System.out.println(TRACE_HEADER + "hashMapIntegratedDefectExternalIdStatus: " + hashMapIntegratedDefectExternalIdStatus);
			
			// +--------------------+-
			// | Status Mapping     |
			// +--------------------+
			// | TMS		XStudio |
			// +--------------------+
			//   N*		-->	check that in XStudio it's New OR Assigned, if NOT, set the XStudio bug to Assigned
			//   SW		-->	check that in XStudio it's Assigned. If NOT, set the XStudio bug to Assigned. If it's assigned to "SoftComputer" assign it to "xxx" and set the "Fix Description" field to "SCC requested further information on <date>"
			//   TW		-->	check that in XStudio it's Resolved. If NOT, set the XStudio bug to Resolved.
			//   C*		-->	check that in XStudio it's Closed. If NOT, set the XStudio bug to Closed and set the "Fix Description" field to "Closed in TMS on <date>"
			//   <else>	-->	check that in XStudio it's Ack OR Resolved, if NOT, set the XStudio bug to Acknowledged
			
			Iterator it = hashMapIntegratedDefectIdExternalId.entrySet().iterator();
		    while (it.hasNext()) {
		        Map.Entry<String, String> pair = (Map.Entry)it.next();
		        String xstudioId = pair.getKey();
		        String starId    = pair.getValue();
		        
		        String xstudioStatus = hashMapIntegratedDefectExternalIdStatus.get(starId);
		        String starStatus    = hashMapTaskIdStatus.get(starId);
		        
		        System.out.println(" -> xstudioId=" + xstudioId + " starId=" + starId + " xstudioStatus=" + xstudioStatus + " starStatus=" + starStatus);
		        
		        if (starStatus.startsWith("N") && (!xstudioStatus.equals(""+INTEGRATED_BTD_STATUS_NEW) && !xstudioStatus.equals(""+INTEGRATED_BTD_STATUS_ASSIGNED))) {
		        	System.out.println(TRACE_HEADER + "==> setting bug status to Assigned...");
		        	CStarGlobals.updateEngine.setIntegratedDefectStatus(xstudioId, ""+INTEGRATED_BTD_STATUS_ASSIGNED);
		        	
		        } else if (starStatus.endsWith("SW") && !xstudioStatus.equals(""+INTEGRATED_BTD_STATUS_ASSIGNED)) {
		        	System.out.println(TRACE_HEADER + "==> setting bug status to Assigned...");
		        	CStarGlobals.updateEngine.setIntegratedDefectStatus(xstudioId, ""+INTEGRATED_BTD_STATUS_ASSIGNED);

		        	String assigneeName = CStarGlobals.getEngine.getDefectIntegrated_AssigneeName(xstudioId);
		        	System.out.println(TRACE_HEADER + "assigneeName = " + assigneeName);
		        	if (assigneeName.equals(IStarConstants.USER_SOFTCOMPUTER)) { // "SoftComputer"
		        		System.out.println(TRACE_HEADER + "==> assigning bug to " + IStarConstants.USER_CLIENT + "...");
		        		CStarGlobals.updateEngine.setIntegratedDefectAssignee(xstudioId, IStarConstants.USER_SOFTCOMPUTER, IStarConstants.USER_CLIENT);
		        	}
		        	
		        	System.out.println(TRACE_HEADER + "==> adding a fix description...");
		        	CStarGlobals.updateEngine.setDefectFixDescription(xstudioId, "SCC requested further information on " + CCalendarUtils.getCurrentTime("GMT"));

		        } else if (starStatus.endsWith("TW") && !xstudioStatus.equals(""+INTEGRATED_BTD_STATUS_RESOLVED)) {
		        	System.out.println(TRACE_HEADER + "==> setting bug status to Resolved...");
		        	CStarGlobals.updateEngine.setIntegratedDefectStatus(xstudioId, ""+INTEGRATED_BTD_STATUS_RESOLVED);
		        	
		        } else if (starStatus.startsWith("C") && !xstudioStatus.equals(""+INTEGRATED_BTD_STATUS_CLOSED)) {
		        	System.out.println(TRACE_HEADER + "==> setting bug status to Resolved...");
		        	CStarGlobals.updateEngine.setIntegratedDefectStatus(xstudioId, ""+INTEGRATED_BTD_STATUS_RESOLVED);

		        	System.out.println(TRACE_HEADER + "==> adding a fix description...");
		        	CStarGlobals.updateEngine.setDefectFixDescription(xstudioId, "Closed in TMS on " + CCalendarUtils.getCurrentTime("GMT"));
		        	
		        } else if (!xstudioStatus.equals(""+INTEGRATED_BTD_STATUS_ACKNOWLEDGED) && !xstudioStatus.equals(""+INTEGRATED_BTD_STATUS_RESOLVED)) {
		        	System.out.println(TRACE_HEADER + "==> setting bug status to Acknowledged...");
		        	CStarGlobals.updateEngine.setIntegratedDefectStatus(xstudioId, ""+INTEGRATED_BTD_STATUS_ACKNOWLEDGED);
		        }
		    }
        }
		
		return new CTriggerStatus(RESULT_SUCCESS, message);
	}

	@Override
	public CTriggerStatus onBugCreated(int id, String name, String description, String stepToReproduce, short reproducibility,
									   int platform, int os,
									   short status, short severity, short priority,
									   int reporter, Vector<Integer> assigneeIds, Vector<Integer> sutIds,
									   Hashtable<Integer, Integer> customFieldIntegerHashtable,
									   Hashtable<Integer, String> customFieldStringHashtable,
									   Hashtable<Integer, String> customFieldBooleanHashtable,
									   Hashtable<Integer, String> customFieldHtmlHashtable,
									   Hashtable<Integer, String> customFieldStringChoiceHashtable
									   ) {
		System.out.println(TRACE_HEADER + "onBugCreated()");
		// XStudio sends a "tc" to TMS (with valid data)
		
		// on Success:
		// - task_id  is saved in ExternalID field
		// - add task_id as prefix in the defect name field
		// on error:
		// - Message is displayed asking the user to create manually the task in TMS and enter its task_id into the defect's external_id in XStudio
		
		//                                      Mandatory	Create	
		//   Client				String			M			Y			"PATHW"
		//   userid				String			M			Y			<ReporterId> if found, "xxx" if no match
		//   Contact			String	11		O			Y			<AssignedToId> if found, "xxx" if no match
		//   Urgency			String	1		O			Y			<See Emergency Mapping>
		//   Data 				String	3800	M			Y			<Name>+<Description>+<StepsToReproduce>
		//   CLSysID			String	20		O			Y			<BugId>
		//   CLSysStatus		String	20		O			Y			<See Status Mapping>
		//   CLSysSeverity		String	10		O						<Severity>
		//   CLSysPriority		String	10		O						<Priority>
		//   CLSysProduct		String	50		O						<SUTName> (must match TMS Product code)
		//   CLSysEnvironment	String	20		M			Y			<customField "Environment"> Trigram as in Environment List ("DEV_SYS_DB", "DEV_SYS_PROD", "DEV", "TRAIN", "UAT", "SIT")
		
		String reporterName = IStarConstants.USER_CLIENT; // by default "xxx"
		String assigneeName = IStarConstants.USER_CLIENT; // by default "xxx"
		
		String xstudioReporterName = CStarGlobals.getEngine.getUserNameFromUserId(""+reporter);
		System.out.println(TRACE_HEADER + "==> xstudio reporter: " + xstudioReporterName);
		if (CStarUtils.existsInArray(xstudioReporterName, IStarConstants.STAR_USERS))
			reporterName = xstudioReporterName;
		System.out.println(TRACE_HEADER + "==> reporter: " + reporterName);
		
		try {
			String xstudioAssigneeName = CStarGlobals.getEngine.getUserNameFromUserId(""+assigneeIds.get(0));
			System.out.println(TRACE_HEADER + "==> cstudio assignee: " + xstudioAssigneeName);
			if (CStarUtils.existsInArray(xstudioAssigneeName, IStarConstants.STAR_USERS))
				assigneeName = xstudioAssigneeName;
		} catch (Exception e) {
		}
		System.out.println(TRACE_HEADER + "==> assignee: " + assigneeName);

		String xstudioSutName = CStarGlobals.getEngine.getSutNameFromSutId(""+sutIds.get(0));
		System.out.println(TRACE_HEADER + "==> sut: " + xstudioSutName); // SUT name in XStudio must be one of those: "AR", "AWR", "BB" etc. (see list in IStarConstants)
		
		String environment = customFieldStringChoiceHashtable.get(IStarConstants.CUSTOMFIELD_ENVIRONMENT_ID); // Id of the combobox custom field holding the environment list ("DEV_SYS_DB", "DEV_SYS_PROD", "DEV", "TRAIN", "UAT", "SIT")
		System.out.println(TRACE_HEADER + "==> environment: " + environment);
		if (environment == null)
			System.err.println("missing custom field holding the environment!");
		
		String urgency = xstudioSeverityAndPriorityToStarUrgency(severity, priority);
		System.out.println(TRACE_HEADER + "==> urgency: " + urgency);
		
		String taskStatus = xstudioStatusToStarStatus(status);
		System.out.println(TRACE_HEADER + "==> taskStatus: " + taskStatus);
		
		System.out.println(TRACE_HEADER + "==> creating the counterpart task...");
		String jsonContent = CStarGlobals.starRestEngine.taskAdd(name + "\\n" + CTextUtils.replaceTabLfAndCrWithSpace(description) + "\\n" + CTextUtils.replaceTabLfAndCrWithSpace(stepToReproduce),  // task description
																 urgency,                                                                                                                             // task urgency
																 reporterName,                                                                                                                        // task reporter
																 assigneeName,                                                                                                                        // task contact
																 ""+id,          // defect id
																 taskStatus,     // defect status
																 ""+severity,    // defect severity (code because max length = 10)
																 ""+priority,    // defect priority
																 xstudioSutName, // product = SUT
																 environment     // environment = custom field
																 );
		
		JSONObject rootObject = new JSONObject(jsonContent);
		boolean success  = rootObject.getBoolean("Success");
		String message   = rootObject.getString("Msg");

		System.out.println(TRACE_HEADER + "success: " + success);
		System.out.println(TRACE_HEADER + "message: " + message);
		if (!success) {
			try {
				JOptionPane.showMessageDialog(null, "Error returned by TMS: \"" + message + "\"\nTask couldn't be created automatically in TMS!\nPlease create manually the task and enter manually it's id in the \"external id\" field in XStudio");
			} catch (HeadlessException e) {
			}
			return new CTriggerStatus(RESULT_FAILURE, message);
		}
		
		// message looks like "Task PATHW-04422 Created"
		String newlyCreatedTaskId = message.split(" ")[1].trim();
		newlyCreatedTaskId = newlyCreatedTaskId.substring(newlyCreatedTaskId.indexOf("-")+1, newlyCreatedTaskId.length());
		
		// set externalId in the xstudio defect
    	System.out.println(TRACE_HEADER + "==> setting external id to the defect...");
    	CStarGlobals.updateEngine.setIntegratedDefectExternalId(""+id, newlyCreatedTaskId);

		// rename the XStudio defect to add the taskId as prefix
    	System.out.println(TRACE_HEADER + "==> adding prefix to the defect...");
    	CStarGlobals.updateEngine.setIntegratedDefectName(""+id, "{TMS_" + newlyCreatedTaskId + "} " + name);
		
		return new CTriggerStatus(RESULT_SUCCESS, "{TMS_" + newlyCreatedTaskId + "} " + name); // we return the new name so that we can use it to dynamically update the tree
	}

	@Override
	public CTriggerStatus onBugUpdated(int id, Vector<CPropertyChange> changeList) {
		System.out.println(TRACE_HEADER + "onBugUpdated()");
		
		// If the defect has just been assigned to "SoftComputer" and current status is "Assigned" or "New":
		// - XStudio sends an actionAdd(taskId, user, "r1")
		
		// If the defect has just been assigned to anyone else than "SoftComputer":
		// - XStudio sends an "actionAdd(taskId, user, "ie")
		
		// If the defect's new Status = "Resolved":
		// - XStudio sends an "actionAdd(taskId, user, "tw")

		// If the defect's new Status = "Assigned" or "New" (re-opened) and if current assignee is "SoftComputer":
		// - XStudio sends an "actionAdd(taskId, user, "r1")

		// If the defect's new Status = "Closed":
		// - XStudio sends an "actionAdd(taskId, user, "c1") (as requested but "c1" not found in the doc though!)

		String taskId = null;
		
		for (int i=0; i<changeList.size(); i++) {
			CPropertyChange propertyChange = changeList.get(i);
			
			// check if the assignee changed
			if (propertyChange.property.equals(""+IStarConstants.ACTION_MODIFY_ASSIGNEDTO)) {
				System.out.println(TRACE_HEADER + "==> Assignee just changed...");

				String newAssigneeId = propertyChange.newValue;
				String newAssigneeName = CStarGlobals.getEngine.getUserNameFromUserId(newAssigneeId);
				System.out.println(TRACE_HEADER + "==> New assignee name: " + newAssigneeId + "/" + newAssigneeName);
				
				// retrieve the defect's external_id
				if (taskId == null) {
					taskId = CStarGlobals.getEngine.getDefectIntegrated_ExternalId(""+id);
					System.out.println(TRACE_HEADER + "==> Task id: " + taskId);
				}

				if (newAssigneeName.equals(IStarConstants.USER_SOFTCOMPUTER)) {
					
					// get the status of the defect
					CDefectDetails defectDetails = CStarGlobals.getEngine.getDefectIntegrated_Details(""+id);
					short currentStatus = defectDetails.status;
					
					if (currentStatus == INTEGRATED_BTD_STATUS_NEW || currentStatus == INTEGRATED_BTD_STATUS_ASSIGNED) {
						System.out.println(TRACE_HEADER + "==> sending a \"r1\"...");
						String jsonContent = CStarGlobals.starRestEngine.actionAdd(taskId,                     // taskID
																				   IStarConstants.USER_CLIENT, // userid
																				   "r1");                      // action
						
						JSONObject rootObject = new JSONObject(jsonContent);
						boolean success  = rootObject.getBoolean("Success");
						String message   = rootObject.getString("Msg");

						System.out.println(TRACE_HEADER + "success: " + success);
						System.out.println(TRACE_HEADER + "message: " + message);
						if (!success) {
							try {
								JOptionPane.showMessageDialog(null, "Error returned by TMS: \"" + message + "\"\nrc couldn't be sent!");
							} catch (HeadlessException e) {
							}
							return new CTriggerStatus(RESULT_FAILURE, message);
						}
					}
					
				} else {
					System.out.println(TRACE_HEADER + "==> sending a \"ie\"...");
					String jsonContent = CStarGlobals.starRestEngine.actionAdd(taskId,                     // taskID
																			   IStarConstants.USER_CLIENT, // userid
																			   "ie");                      // action
					
					JSONObject rootObject = new JSONObject(jsonContent);
					boolean success  = rootObject.getBoolean("Success");
					String message   = rootObject.getString("Msg");

					System.out.println(TRACE_HEADER + "success: " + success);
					System.out.println(TRACE_HEADER + "message: " + message);
					if (!success) {
						try {
							JOptionPane.showMessageDialog(null, "Error returned by TMS: \"" + message + "\"\nie couldn't be sent!");
						} catch (HeadlessException e) {
						}
						return new CTriggerStatus(RESULT_FAILURE, message);
					}
				}
			}
			
			// check if the status changed
			if (propertyChange.property.equals(""+IStarConstants.ACTION_MODIFY_STATUS)) {
				System.out.println(TRACE_HEADER + "==> status just changed...");

				String newStatus = propertyChange.newValue;
				System.out.println(TRACE_HEADER + "==> New status: " + newStatus);

				if (newStatus.equals(""+INTEGRATED_BTD_STATUS_RESOLVED)) {
					
					String assigneeName = CStarGlobals.getEngine.getDefectIntegrated_AssigneeName(""+id);
					
					// retrieve the defect's external_id
					if (taskId == null) {
						taskId = CStarGlobals.getEngine.getDefectIntegrated_ExternalId(""+id);
						System.out.println(TRACE_HEADER + "==> Task id: " + taskId);
					}

					System.out.println(TRACE_HEADER + "==> sending a \"tw\"...");
					String jsonContent = CStarGlobals.starRestEngine.actionAdd(taskId,                     // taskID
																			   IStarConstants.USER_CLIENT, // userid
																			   "tw");                      // action
					
					JSONObject rootObject = new JSONObject(jsonContent);
					boolean success  = rootObject.getBoolean("Success");
					String message   = rootObject.getString("Msg");

					System.out.println(TRACE_HEADER + "success: " + success);
					System.out.println(TRACE_HEADER + "message: " + message);
					if (!success) {
						try {
							JOptionPane.showMessageDialog(null, "Error returned by TMS: \"" + message + "\"\ntw couldn't be sent!");
						} catch (HeadlessException e) {
						}
						return new CTriggerStatus(RESULT_FAILURE, message);
					}

				} else if (newStatus.equals(""+INTEGRATED_BTD_STATUS_NEW) || newStatus.equals(""+INTEGRATED_BTD_STATUS_ASSIGNED)) { // reopened
					
					String assigneeName = CStarGlobals.getEngine.getDefectIntegrated_AssigneeName(""+id);
					if (assigneeName.equals(IStarConstants.USER_SOFTCOMPUTER)) {
					
						// retrieve the defect's external_id
						if (taskId == null) {
							taskId = CStarGlobals.getEngine.getDefectIntegrated_ExternalId(""+id);
							System.out.println(TRACE_HEADER + "==> Task id: " + taskId);
						}
	
						System.out.println(TRACE_HEADER + "==> sending a \"r1\"...");
						String jsonContent = CStarGlobals.starRestEngine.actionAdd(taskId,                     // taskID
																				   IStarConstants.USER_CLIENT, // userid
																				   "r1");                      // action
						
						JSONObject rootObject = new JSONObject(jsonContent);
						boolean success  = rootObject.getBoolean("Success");
						String message   = rootObject.getString("Msg");
	
						System.out.println(TRACE_HEADER + "success: " + success);
						System.out.println(TRACE_HEADER + "message: " + message);
						if (!success) {
							try {
								JOptionPane.showMessageDialog(null, "Error returned by TMS: \"" + message + "\"\nrc couldn't be sent!");
							} catch (HeadlessException e) {
							}
							return new CTriggerStatus(RESULT_FAILURE, message);
						}
					}
					
				} else if (newStatus.equals(""+INTEGRATED_BTD_STATUS_CLOSED)) {
					
					// retrieve the defect's external_id
					if (taskId == null) {
						taskId = CStarGlobals.getEngine.getDefectIntegrated_ExternalId(""+id);
						System.out.println(TRACE_HEADER + "==> Task id: " + taskId);
					}

					System.out.println(TRACE_HEADER + "==> sending a \"c1\"...");
					String jsonContent = CStarGlobals.starRestEngine.actionAdd(taskId,                     // taskID
																			   IStarConstants.USER_CLIENT, // userid
																			   "c1");                      // action
					
					JSONObject rootObject = new JSONObject(jsonContent);
					boolean success  = rootObject.getBoolean("Success");
					String message   = rootObject.getString("Msg");

					System.out.println(TRACE_HEADER + "success: " + success);
					System.out.println(TRACE_HEADER + "message: " + message);
					if (!success) {
						try {
							JOptionPane.showMessageDialog(null, "Error returned by TMS: \"" + message + "\"\nc1 couldn't be sent!");
						} catch (HeadlessException e) {
						}
						return new CTriggerStatus(RESULT_FAILURE, message);
					}

				}
			}
		}
		
		return new CTriggerStatus();
	}

	@Override
	public CTriggerStatus onBugsDeleted(Vector<Integer> ids) {
		System.out.println(TRACE_HEADER + "onBugsDeleted()");
		// Not authorized
		return new CTriggerStatus();
	}

	@Override
	public CTriggerStatus onBugsMoved(Vector<Integer> ids, int folderId) {
		System.out.println(TRACE_HEADER + "onBugsMoved()");
		return new CTriggerStatus();
	}

	// +------------------------------------+
	// | Terminate                          |
	// +------------------------------------+
	
	@Override
	public CTriggerStatus terminate() {
		System.out.println(TRACE_HEADER + "terminate()");
		return new CTriggerStatus();
	}

	
	// +==============================================================+
	// | Utilities                                                      |
	// +==============================================================+

	// +--------------------+-
	// | Status Mapping     |
	// +--------------------+
	// | TMS		XStudio |
	// +--------------------+
	//   N		-->	check that in XStudio it's New OR Assigned, if NOT, set the XStudio bug to Assigned
	//   *W		-->	check that in XStudio it's Assigned 
	//   C*		-->	check that in XStudio it's Closed
	//   <Any>	-->	check that in XStudio it's Ack OR Resolved, if NOT, set the XStudio bug to Ack
	public String xstudioStatusToStarStatus(short status) {
		if (status == INTEGRATED_BTD_STATUS_NEW || status == INTEGRATED_BTD_STATUS_ASSIGNED) {
			return "N";
			
		} else if (status == INTEGRATED_BTD_STATUS_CLOSED) {
			return "Cl";
			
		} else if (status == INTEGRATED_BTD_STATUS_ACKNOWLEDGED || status == INTEGRATED_BTD_STATUS_RESOLVED) {
			return "?";
			
		} else {
			return "?";
		}
	}
	
	// +-------------------------------------------------+-
	// | Emergency Mapping (XStudio -> TMS)              |
	// +-------------------------------------------------+
	// | XStudio Severity	XStudio priority	TMS      |
	// +-------------------------------------------------+
	//   Minor				<Any>				R
	//   Enhanc., Cosm.		<Any>				S
	//   Major				Low, Normal			P
	//   Major				High				U
	//   Blocking			<Any>				U
	
	public String xstudioSeverityAndPriorityToStarUrgency(short severity, short priority) {
		if (severity == INTEGRATED_BTD_SEVERITY_MINOR) {
			return "R";
			
		} else if (severity == INTEGRATED_BTD_SEVERITY_ENHANCEMENT || severity == INTEGRATED_BTD_SEVERITY_COSMETIC) {
			return "S";
			
		} else if (severity == INTEGRATED_BTD_SEVERITY_MAJOR && (priority == INTEGRATED_BTD_PRIORITY_LOW || priority == INTEGRATED_BTD_PRIORITY_NORMAL)) {
			return "P";
			
		} else if (severity == INTEGRATED_BTD_SEVERITY_MAJOR && priority == INTEGRATED_BTD_PRIORITY_HIGH) {
			return "U";
			
		} else if (severity == INTEGRATED_BTD_SEVERITY_BLOCKING) {
			return "U";
			
		} else {
			return "P";
		}
	}
	
	
	// +-------------------------------------------------+-
	// | Emergency Mapping (TMS --> XStudio)             |
	// +-------------------------------------------------+
	// | TMS		XStudio Severity	XStudio priority |
	// +-------------------------------------------------+
	//   R			Minor				Normal
	//   S			Enhancement			Normal
	//   P			Major				Normal
	//   U			Major				High

	public short starUrgencyToXStudioSeverity(String urgency) {
		if (urgency.equals("R")) {
			return INTEGRATED_BTD_SEVERITY_MINOR;
			
		} else if (urgency.equals("S")) {
			return INTEGRATED_BTD_SEVERITY_ENHANCEMENT;
			
		} else if (urgency.equals("P") || urgency.equals("U")) {
			return INTEGRATED_BTD_SEVERITY_MAJOR;
			
		} else {
			return INTEGRATED_BTD_SEVERITY_MAJOR;
		}
	}

	public short starUrgencyToXStudioPriority(String urgency) {
		if (urgency.equals("R") || urgency.equals("S") || urgency.equals("P")) {
			return INTEGRATED_BTD_PRIORITY_NORMAL;
			
		} else if (urgency.equals("U")) {
			return INTEGRATED_BTD_PRIORITY_HIGH;
			
		} else {
			return INTEGRATED_BTD_PRIORITY_NORMAL;
		}
	}

	
	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+
/*
	public static void main(String[] args) {
		CTriggerImpl toto = new CTriggerImpl();
		toto.initialize();
		
		String jsonContent = "";

		// *********** taskGet *************
		jsonContent = toto.starRestEngine.taskGet("03271"); // sometimes we'll have to pass an extra userId = "xxx" in the params sections

		// *********** qryTasks *************
		jsonContent = toto.starRestEngine.qryTasks("20180201000001");

		JSONObject rootObject = new JSONObject(jsonContent);
		boolean success  = rootObject.getBoolean("Success");
		String message   = rootObject.getString("Msg");

		System.out.println(TRACE_HEADER + "success: " + success);
		System.out.println(TRACE_HEADER + "message: " + message);

	}
*/
}
