/*
+----------------------------------------------------------------------+
|                    Class: CConnectionManager                         |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
+----------------------------------------------------------------------+
*/
package com.xqual.xtrigger.star.rest;

import java.io.DataOutputStream;
import java.net.HttpURLConnection;
import java.net.URL;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;

import com.xqual.xcommon.utils.CFileUtils;

public class CStarRestConnectionManager {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static HostnameVerifier yesHostnameVerifier = null;
	static TrustManager     yesTrustManager     = null;
	static SSLSocketFactory yesSslFactory       = null;
	
	public HttpURLConnection connection = null;
	
	private final String baseUrl; // final base URL

	private final String username;
	
	private URL url; // complete last URL
	
	public boolean httpsEnabled = false;
	
	private int responseCode;
	private String responseMessage;
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	static {
        // prepare Verifiers that will accept ANY certificate from ANY host
        yesHostnameVerifier = new CStarYesHostnameVerifier();
        yesTrustManager     = new CStarYesTrustManager();
	}
	
	public CStarRestConnectionManager(String baseUrl, String username) {
	    this.baseUrl  = baseUrl;  // https://www.acme.com
		this.username = username;

	    try {
	    	url = new URL(baseUrl);
		} catch (Throwable t) {
			t.printStackTrace();
			return;
		}
	    
	    // ------------------------------
	    // configure the HTTPS connection
	    // ------------------------------
	    if (url.getProtocol().equalsIgnoreCase("https")) {
			System.out.println("the STAR server is an https/SSL server");
		    httpsEnabled = true;
		    
		    // USE THE SAME SETTINGS THAN FOR DATABASE (database.connection.accept_any_certificate)
		    //if (CStudioGlobals.applicationConfProperties.isAcceptAnyCertificate()) { // the user accepts ANY certificate
		    	System.out.println("==> accepting ANY certificate");
		        // initiate the SSL context and prepare trust manager that will accept ANY certificate
		        TrustManager[] yesTrustManagers = new TrustManager[] {yesTrustManager};
		        try {
					SSLContext yesSslcontext = SSLContext.getInstance("TLS");
					yesSslcontext.init(null, yesTrustManagers, null); // key manager, trust manager, secure random manager
					SSLSocketFactory yesSslFactory = yesSslcontext.getSocketFactory();
					
				    HttpsURLConnection.setDefaultSSLSocketFactory(yesSslFactory);
				    HttpsURLConnection.setDefaultHostnameVerifier(yesHostnameVerifier);
		        } catch (Exception e) {
				}
		        
		    /*} else {
		    	System.out.println(" ==> accepting only certificate matching a certificate from the local/private keystore");
		    	HttpsURLConnection.setDefaultSSLSocketFactory(CStudioGlobals.xstudioSslFactory); // this will trust the server only if its certificates has been added in the conf
		    }*/

	    // -----------------------------
	    // configure the HTTP connection
	    // -----------------------------
	    } else {
	    	System.out.println("the STAR server is an http server");
		    httpsEnabled = false;
	    }
	    
	    System.out.println("base url = " + baseUrl);
	    System.out.println("username = " + username);
	    System.out.println("password = ******" /*+ password*/);
	}
	
	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	public String getData(String urlPostfix, String jsonData) {
		try {
			// data to pass in the body
			byte[] postData       = jsonData.getBytes();
			int    postDataLength = postData.length;
			
	        // prepare the connection
	        url = new URL(baseUrl + "/" + urlPostfix);
	        System.out.println("connecting to url \"" + url + "\" with account: " + username);
	        
	        // the connection may already be opened so we catch the IllegalStateException "Already Connected"
	        try {
				if (httpsEnabled) {
					connection = (HttpsURLConnection)url.openConnection(); // HttpsURLConnection extends HttpURLConnection

				} else if (url.getProtocol().equalsIgnoreCase("http")) {
					connection = (HttpURLConnection)url.openConnection();
				}
	        
				connection.setDoOutput(true);
				//connection.setInstanceFollowRedirects(false);
	            connection.setRequestMethod("POST");
	            connection.setRequestProperty("Content-Type", "application/json"); 
	            //connection.setRequestProperty("charset", "utf-8");
	            connection.setRequestProperty("Content-Length", Integer.toString(postDataLength));
	            connection.setUseCaches(false);

	            
	            /*
	            http://stackoverflow.com/questions/17278303/basic-authentication-fails-with-glassfish
	            Oracle has enabled caching of HTTP responses by default with JDK7 in Web Start: 
				Caching enabled by default: Caching of network content for application code running in Web Start mode is now enabled by default. 
				This allows application improved performance and consistency with applet execution mode. 
				To ensure the latest copy of content is used, the application can use URLConnection.setUseCaches(false) or request header Cache-Control values no-cache/no-store.
				*/
	            // maybe we should try to re-enable cache after the first connection?
		        connection.setRequestProperty("Cache-Control", "no-cache");
		        connection.setRequestProperty("Cache-Control", "no-store");
		        connection.setRequestProperty("Connection", "keep-alive");
		        
	        } catch (IllegalStateException ise) {
				System.err.println("Already connected\n");
			}
	        
	        // ----------- CONNECTION -----------
            try (DataOutputStream wr = new DataOutputStream(connection.getOutputStream())) {
            	wr.write( postData );
	        }

	        // store the code and the message so that the caller can retrieve this
	        responseCode = connection.getResponseCode();
	        responseMessage = connection.getResponseMessage();
	        
	        if (responseCode != 200) {
	        	System.err.println("Server response: " + responseCode + " (" + responseMessage + ")");
	        } else {
	        	System.out.println("Server response: " + responseCode + " (" + responseMessage + ")");
	        }
	        
	        if (responseCode != 200) {
	        	//String output = CFileUtils.streamToString(connection.getErrorStream());
		        //System.err.println("ERROR=" + output);
				try {
					System.err.println("Disconnecting...");
					connection.disconnect();
				} catch (Exception e) {
					System.err.println("Couldn't disconnect...");
				}
	        	return null;
	        }
	        
	        String output = CFileUtils.streamToString(connection.getInputStream());
	        //System.err.println("OUTPUT=" + output);
	        //connection.disconnect();
	        return output;
	        
		} catch (Throwable t) {
			System.err.println("Exception while communicating with STAR server: " + t);
			try {
				System.err.println("Disconnecting...");
				connection.disconnect();
			} catch (Exception e) {
				System.err.println("Couldn't disconnect...");
			}
			return null;
		}
	}	
	
	public int getResponseCode() {
		return responseCode;
	}

	public String getResponseMessage() {
		return responseMessage;
	}
	
	public String getBaseUrl() {
		return baseUrl;
	}

	public URL getUrl() {
		return url;
	}

}
