/*
+----------------------------------------------------------------------+
|                    Class: CStarRestEngine                            |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
+----------------------------------------------------------------------+
*/
package com.xqual.xtrigger.star.rest;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Calendar;
import java.util.TimeZone;

import com.xqual.xcommon.utils.CConversionUtils;
import com.xqual.xtrigger.star.CStarCrypto;
import com.xqual.xtrigger.star.IStarConstants;

public class CStarRestEngine implements IStarConstants{

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	final CStarRestConnectionManager starConnectionManager;
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CStarRestEngine() {
		System.out.println("instantiate the STAR REST engine " + URL + "...");
		starConnectionManager = new CStarRestConnectionManager(URL, USER_ID);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	public String qryTasks(String dateFrom) {
		String datetime = getCurrentDateTime();
		String dateTo = datetime;

		System.out.println("dateFrom = " + dateFrom);
		System.out.println("dateTo   = " + dateTo);
		
		String toBeHashed = USER_ID + datetime + "Client" + CLIENT + "date_from" + dateFrom + "date_to" + dateTo + PRIVATE_KEY;
		System.out.println("toBeHashed = " + toBeHashed);
		String hash = CConversionUtils.byteArrayToByteArrayString(CStarCrypto.digestMd5(toBeHashed)).toLowerCase();
		System.out.println("hash       = " + hash);
		
		StringBuffer restRequestBuffer = new StringBuffer();
		restRequestBuffer.append("{");
		
		restRequestBuffer.append("\"header\":{");
		restRequestBuffer.append("\"userid\":\"" + USER_ID + "\",");
		restRequestBuffer.append("\"datetime\":\"" + datetime + "\",");
		restRequestBuffer.append("\"HASH\":\"" + hash + "\"");
		restRequestBuffer.append("},");
		
		restRequestBuffer.append("\"params\":{");
		restRequestBuffer.append("\"Client\":\"" + CLIENT + "\",");
		restRequestBuffer.append("\"date_from\":\"" + dateFrom + "\",");
		restRequestBuffer.append("\"date_to\":\"" + dateTo + "\"");
		restRequestBuffer.append("},");
		
		restRequestBuffer.append("\"data\":{}");
		restRequestBuffer.append("}");
		System.out.println("json req.  = " + restRequestBuffer.toString());


    	String data = starConnectionManager.getData("qry/tasks", restRequestBuffer.toString());
		System.err.println("data = " + data);
		System.err.println("FINISHED");
		return data;
	}

	public String taskGet(String taskId) {
		String datetime = getCurrentDateTime();
		System.out.println("datetime   = " + datetime);
		
		String toBeHashed = USER_ID + datetime + "Client" + CLIENT + "TaskID" + taskId + PRIVATE_KEY;
		System.out.println("toBeHashed = " + toBeHashed);
		String hash = CConversionUtils.byteArrayToByteArrayString(CStarCrypto.digestMd5(toBeHashed)).toLowerCase();
		System.out.println("hash       = " + hash);
		
		StringBuffer restRequestBuffer = new StringBuffer();
		restRequestBuffer.append("{");
		
		restRequestBuffer.append("\"header\":{");
		restRequestBuffer.append("\"userid\":\"" + USER_ID + "\",");
		restRequestBuffer.append("\"datetime\":\"" + datetime + "\",");
		restRequestBuffer.append("\"HASH\":\"" + hash + "\"");
		restRequestBuffer.append("},");
		
		restRequestBuffer.append("\"params\":{");
		restRequestBuffer.append("\"Client\":\"" + CLIENT + "\",");
		restRequestBuffer.append("\"TaskID\":\"" + taskId + "\"");
		restRequestBuffer.append("},");
		
		restRequestBuffer.append("\"data\":{}");
		restRequestBuffer.append("}");
		System.out.println("json req.  = " + restRequestBuffer.toString());


    	String data = starConnectionManager.getData("task/get", restRequestBuffer.toString());
		System.err.println("data = " + data);
		System.err.println("FINISHED");
		return data;
	}

	
	public String taskAdd(String taskDescription, String taskUrgency, String taskReporter, String taskAssignee, 
						  String defectId, String defectStatus, String defectSeverity, String defectPriority, String defectProduct, String defectEnvironment) {
		String datetime = getCurrentDateTime();
		System.out.println("datetime   = " + datetime);
		
		String toBeHashed = USER_ID + datetime + 
							"Client"           + CLIENT + 
							"userid"           + taskReporter + 
							"Contact"          + taskAssignee +
							"Environment"      + defectEnvironment +
							"Product"          + defectProduct +
							"Urgency"          + taskUrgency +
							"CLSysID"          + defectId +
							"CLSysStatus"      + defectStatus +
							//"CLSysSeverity"    + defectSeverity +
							//"CLSysPriority"    + defectPriority +
							"CLSysProduct"     + defectProduct +
							"CLSysEnvironment" + defectEnvironment +
							PRIVATE_KEY;
		
		System.out.println("toBeHashed = " + toBeHashed);
		String hash = CConversionUtils.byteArrayToByteArrayString(CStarCrypto.digestMd5(toBeHashed)).toLowerCase();
		System.out.println("hash       = " + hash);
		
		StringBuffer restRequestBuffer = new StringBuffer();
		restRequestBuffer.append("{");
		restRequestBuffer.append("\"header\":{");
		restRequestBuffer.append("\"userid\":\""   + USER_ID + "\",");
		restRequestBuffer.append("\"datetime\":\"" + datetime + "\",");
		restRequestBuffer.append("\"HASH\":\""     + hash + "\"");
		restRequestBuffer.append("},");
		
		restRequestBuffer.append("\"params\":{");
		restRequestBuffer.append("\"Client\":\""           + CLIENT + "\",");
		restRequestBuffer.append("\"userid\":\""           + taskReporter + "\",");
		restRequestBuffer.append("\"Contact\":\""          + taskAssignee + "\",");
		restRequestBuffer.append("\"Environment\":\""      + defectEnvironment + "\",");
		restRequestBuffer.append("\"Product\":\""          + defectProduct + "\",");
		restRequestBuffer.append("\"Urgency\":\""          + taskUrgency + "\",");
		restRequestBuffer.append("\"CLSysID\":\""          + defectId + "\",");
		restRequestBuffer.append("\"CLSysStatus\":\""      + defectStatus + "\",");
		//restRequestBuffer.append("\"CLSysSeverity\":\""    + defectSeverity + "\",");
		//restRequestBuffer.append("\"CLSysPriority\":\""    + defectPriority + "\",");
		restRequestBuffer.append("\"CLSysProduct\":\""     + defectProduct + "\",");
		restRequestBuffer.append("\"CLSysEnvironment\":\"" + defectEnvironment + "\""); 
		restRequestBuffer.append("},");
		
		restRequestBuffer.append("\"data\":{");
		restRequestBuffer.append("\"TaskDescr\":\"" + taskDescription + "\""); 
		restRequestBuffer.append("}");
		
		restRequestBuffer.append("}");
		System.out.println("json req.  = " + restRequestBuffer.toString());

    	String data = starConnectionManager.getData("task/add", restRequestBuffer.toString());
		System.err.println("data = " + data);
		System.err.println("FINISHED");
		return data;
	}

	
	public String actionAdd(String taskId, String userId, String action) {
		String datetime = getCurrentDateTime();
		System.out.println("datetime   = " + datetime);
		
		String toBeHashed = USER_ID + datetime + 
							"Client"           + CLIENT + 
							"TaskID"           + taskId +
							"userid"           + userId + 
							"ActionCode"       + action +
							PRIVATE_KEY;
		
		System.out.println("toBeHashed = " + toBeHashed);
		String hash = CConversionUtils.byteArrayToByteArrayString(CStarCrypto.digestMd5(toBeHashed)).toLowerCase();
		System.out.println("hash       = " + hash);
		
		StringBuffer restRequestBuffer = new StringBuffer();
		restRequestBuffer.append("{");
		restRequestBuffer.append("\"header\":{");
		restRequestBuffer.append("\"userid\":\""   + USER_ID + "\",");
		restRequestBuffer.append("\"datetime\":\"" + datetime + "\",");
		restRequestBuffer.append("\"HASH\":\""     + hash + "\"");
		restRequestBuffer.append("},");
		
		restRequestBuffer.append("\"params\":{");
		restRequestBuffer.append("\"Client\":\""           + CLIENT + "\",");
		restRequestBuffer.append("\"TaskID\":\""           + taskId + "\",");
		restRequestBuffer.append("\"userid\":\""           + userId + "\",");
		restRequestBuffer.append("\"ActionCode\":\""       + action + "\"");
		restRequestBuffer.append("},");
		
		restRequestBuffer.append("\"data\":{");
		restRequestBuffer.append("}");
		
		restRequestBuffer.append("}");
		System.out.println("json req.  = " + restRequestBuffer.toString());

    	String data = starConnectionManager.getData("action/add", restRequestBuffer.toString());
		System.err.println("data = " + data);
		System.err.println("FINISHED");
		return data;
	}
	
	// +==============================================================+
	// | Utils                                                        |
	// +==============================================================+

	private String getCurrentDateTime() {
		Calendar now = Calendar.getInstance(TimeZone.getTimeZone("GMT")); // Must be GMT for STAR
		return calendarToString(now);
	}

	private static String calendarToString(Calendar calendar) {
    	if (calendar == null)
    		return "";

        NumberFormat twoDigitsFormatter = new DecimalFormat("00");
        return calendar.get(Calendar.YEAR) + 
               twoDigitsFormatter.format(calendar.get(Calendar.MONTH)+1) + // first MONTH == 0
               twoDigitsFormatter.format(calendar.get(Calendar.DAY_OF_MONTH)) + 
               twoDigitsFormatter.format(calendar.get(Calendar.HOUR_OF_DAY)) + 
               twoDigitsFormatter.format(calendar.get(Calendar.MINUTE)) + 
               twoDigitsFormatter.format(calendar.get(Calendar.SECOND));
    }
}

