/*
+----------------------------------------------------------------------+
|                    Class: CSqlCreateEngine                           |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
+----------------------------------------------------------------------+
*/
package com.xqual.xtrigger.star.sql;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Calendar;

import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.IConstantsSql;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CSqlUtils;
import com.xqual.xcommon.utils.CTextUtils;
import com.xqual.xtrigger.CTriggerGlobals;
import com.xqual.xtrigger.star.CStarGlobals;
import com.xqual.xtrigger.star.CStarUtils;
import com.xqual.xtrigger.star.IStarConstants;


/**
 * The <code>CSqlConnector</code> class defines the connector to send/receive some SQL requests/responses
 * @author egavaldo
 */
public class CStarSqlCreateEngine implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+


    // simulate pointer on functions
    private final static short FUNCTION_INSERT   = 1;

    
    // could keep it unique and replace the appropriate field names with ? for oracle  
    public final static String MYSQL_INSERT_DEFECTINTEGRATEDCONTENT  = "INSERT INTO table_def_int_data (defect_id, defect_description, priority, severity, status, " +
    		                                                           "                                step_to_reproduce, reproducibility, platform_id, os_id, fix_target_date, fix_target_version, completion_percentage, " +
    		                                                           "                                fix_type, fix_description, fix_patches, date_submitted, revision) " +
	                                                                   "VALUES (#defect_id#, #defect_description#, #priority#, #severity#, #status#, " +
	                                                                   "        #step_to_reproduce#, #reproducibility#, #platform_id#, #os_id#, #fix_target_date#, #fix_target_version#, #completion#, " +
	                                                                   "        #fix_type#, #fix_description#, #fix_patches#, #date_submitted#, #revision#)";

    
    public final static String INSERT_CONTENT_TIMETAG	=	"INSERT INTO #table_name#_data_tag (data_id, tag_id) " +
    														"VALUES (#data_id#, #tag_id#)";
    

    public final static String INSERT_CHANGE = "INSERT INTO table_change (type, id, date_submitted, user_id, action, information)" + // entry_id is generated
                                               "VALUES (#type#, #id#, #date_submitted#, #user_id#, #action#, #information#)";
    
    
    public final static String MYSQL_AFFECT_DEFECT_ASSIGNEDTO =	"INSERT INTO table_def_int_assigned_to(defect_id, assigned_to) " +
																"VALUES (#defect_id#, #assigned_to#)";



	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CStarSqlCreateEngine() {
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

    public int insertDefectIntegratedContent(String defectId, String defectDescription, String priority, String severity,
                                             String status, String stepToReproduce, String reproducibility, String platformId, 
                                             String osId, String fixTargetDate, String fixTargetVersion, String completion,
                                             String fixType, String fixDescription, String fixPatches,
										     String newRevision) {
    	System.out.println("[Create] inserting defect content for defect " + defectId + "...");
    	
        Calendar rightNow = CCalendarUtils.getTimezonedInstance();
        
        return executeSQLQuery(FUNCTION_INSERT,
        		               CSqlUtils.computeStatement(MYSQL_INSERT_DEFECTINTEGRATEDCONTENT, new String[][]{{"#defect_id#", defectId},
					                                                                                           {"#defect_description#", "'" + CStarUtils.escapeStringForDatabase(defectDescription) + "'"},
					                                                                                           {"#priority#", priority},
					                                                                                           {"#severity#", severity},
					                                                                                           {"#status#", status},
					                                                                                           {"#step_to_reproduce#", "'" + CStarUtils.escapeStringForDatabase(stepToReproduce) + "'"},
					                                                                                           {"#reproducibility#", reproducibility},
					                                                                                           {"#platform_id#", platformId},
					                                                                                           {"#os_id#", osId},
					                                                                                           {"#fix_target_date#", CSqlUtils.dateTime(fixTargetDate, IConstantsSql.DATABASE_MYSQL)},
					                                                                                           {"#fix_target_version#", "'" + CStarUtils.escapeStringForDatabase(fixTargetVersion) + "'"},
					                                                                                           {"#completion#", completion},
					                                                                                           {"#fix_type#", "'" + fixType + "'"},
					                                                                                           {"#fix_description#", "'" + CStarUtils.escapeStringForDatabase(fixDescription) + "'"},
					                                                                                           {"#fix_patches#", "'" + CStarUtils.escapeStringForDatabase(fixPatches) + "'"},
					                                                                                           {"#date_submitted#", CSqlUtils.dateTime(CCalendarUtils.calendarToString(rightNow), IConstantsSql.DATABASE_MYSQL)},
					                                                                                           {"#revision#", newRevision},
					                                                                                           }),
					                "data_id");
	
    }
    
    public int updateDefectContentLatestTimetag(String defectId, String dataId) {
    	return updateContentLatestTimetag("table_def_int", "defect_id", defectId, dataId);
    }

    public int updateContentLatestTimetag(String tableName, String referenceFieldName, String referenceFieldValue, String dataId) {
    	System.out.println("[Create] moving timetag to data_id=" + dataId + " for " + referenceFieldName + "=" + referenceFieldValue + "...");

        // delete the LATEST timetag on this object
        CStarGlobals.deleteEngine.deleteAllLatest(tableName, referenceFieldName, referenceFieldValue);

        // insert the LATEST timetag on the latest data version
        return insertContentTimetag(tableName, dataId, ""+IStarConstants.TIMETAG_ID_LATEST);
    }

    public int insertContentTimetag(String tableName, String dataId, String timetagId) {
    	System.out.println("[Create] inserting timetag " + timetagId + " to data_id=" + dataId + " on " + tableName + "...");
        return executeSQLQuery(FUNCTION_INSERT,
        					   CSqlUtils.computeStatement(INSERT_CONTENT_TIMETAG, new String[][]{{"#table_name#", tableName},
        							   															 {"#data_id#", dataId},
        							   															 {"#tag_id#", timetagId}}),
        					   null);   // no id is generated so we can't test this, or we would need to add an auto-increment id on the xxx_tag table
    }
    
    public int insertChange(String type, String id, String date, String user_id, String action, String information) {
    	System.out.println("[Create] inserting change on " + type + " " + id + "...");
        
        // ensure the information fields is not larger than 255 characters
        if (information == null)
        	information = "";
        else if (information.length() >= 254)
        	information = CTextUtils.left(information, 254);
        
        return executeSQLQuery(FUNCTION_INSERT,
        		               CSqlUtils.computeStatement(INSERT_CHANGE, new String[][]{{"#type#", type},
                                                                                        {"#id#", id},
                                                                                        {"#date_submitted#", CSqlUtils.dateTime(date, IConstantsSql.DATABASE_MYSQL)},
											                                            {"#user_id#", user_id},
											                                            {"#action#", action},
											                                            {"#information#", "'" + CStarUtils.escapeStringForDatabase(information) + "'"}}),
								"id");
    }

    
    public int affectDefectAssignedTo(String assigneeId, String defectId) {
    	System.out.println("[Create] affecting user " + assigneeId + " as assignedTo of defect " + defectId + "...");

        // delete former affectations
        int result = CStarGlobals.deleteEngine.deleteDefectAssignedTo(defectId);
        if (result == -1) {
        	System.out.println("[Create] couldn't delete assignedTo...");
        	return -1;
        }

        // add new affectations
        return insertDefectAssignedTo(assigneeId, defectId);
    }
    public int insertDefectAssignedTo(String assigneeId, String defectId) {
    	System.out.println("[Create] inserting user " + assigneeId + " as assignedTo of defect " + defectId + "...");
    	return executeSQLQuery(FUNCTION_INSERT, MYSQL_AFFECT_DEFECT_ASSIGNEDTO.replaceAll("#defect_id#", defectId)
    																		  .replaceAll("#assigned_to#", assigneeId),
    						   null);
    }
    
	/**
	 * return the result of a query execution
	 */
	public int executeSQLQuery(short functionId, String query, String idName) {
        int result = 0;
		Connection connection = null;
		Statement statement = null;
		ResultSet resultSet = null;
		try {
			connection = CTriggerGlobals.applicationConnectionFactory.getConnection();
			statement = connection.createStatement();

			switch (functionId) {

				case FUNCTION_INSERT:
					System.out.println("   > executing query FUNCTIONFUNCTION_INSERT_UPDATE...");
                    // insert a new row
                    int insertedNb = statement.executeUpdate(query);
                    System.out.println("inserted: " + insertedNb);
                    if (insertedNb < 1) {
                    	System.out.println("Query: " + query);
                        break;
                    }
                    
                    // get the created row id
                    // For LAST_INSERT_ID(), the most recently generated ID is maintained in the server on a <u>per-connection basis</u>.
                    // It is not changed by another client. 
                    // It is not even changed if you update another AUTO_INCREMENT column with a nonmagic value (that is, a value that is not NULL and not 0). 
                    // Using LAST_INSERT_ID() and AUTO_INCREMENT columns simultaneously from multiple clients is perfectly valid. 
                    // Each client will receive the last inserted ID for the last statement that THIS client executed.
                    if (idName == null) {
                    	result = 123; // used when there is no auto-increment id generated
                    } else {
	                    resultSet = statement.executeQuery("SELECT LAST_INSERT_ID();");
	                    if (resultSet.next()) {
	                        result = resultSet.getInt(1);
                        }
                    }
					break;

				default:
					break;
			}

			if (statement != null)
				try {
					statement.close();
				} catch (SQLException e) {
					result = -1;
					System.err.println("SQLException: " + e.getMessage());
				}
		} catch (SQLException e) {
            result = -1;
            System.err.println("SQLException: " + e.getMessage());
            System.err.println("Query: " + query);
			if (e instanceof java.sql.SQLIntegrityConstraintViolationException) { // probably the item being edited (insert of a 'content') does not exist anymore because deleted by someone else while we were editing it
				System.err.println("the item being edited very likely does not exist anymore");
			} else {
				e.printStackTrace();
			}
        } catch (Exception e) { // probably, the SQL connection has been lost
			result = -1;
			System.err.println("sql connection lost");
			e.printStackTrace();

		} finally {
			if (statement != null)
				try {
					statement.close();
				} catch (SQLException e) {
					result = -1;
					System.err.println("SQLException: " + e.getMessage());
				}
		}
		CTriggerGlobals.applicationConnectionFactory.freeConnection(connection);

		System.err.println("result (id inserted, nb of rows inserted etc.): " + result);
        return result;
	}

	// +==============================================================+
	// | Utilities                                                    |
	// +==============================================================+

	public static String getCondition_Revision_DefInt(String timetagId) {
		return getCondition_Revision("table_def_int", "defect_id", timetagId);
	}
	
	public static String getCondition_Revision(String table, String field, String timetagId) {
		StringBuffer outputBuffer = new StringBuffer();
		outputBuffer.append("AND " + table + "." + field + "=" + table + "_data." + field + " ");
		outputBuffer.append("AND " + table + "_data.data_id=" + table + "_data_tag.data_id ");
		outputBuffer.append("AND " + table + "_data_tag.tag_id=" + timetagId + " ");
		return outputBuffer.toString();
	}
}
