/*
+----------------------------------------------------------------------+
|                    Class: CSqlDeleteEngine                           |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
+----------------------------------------------------------------------+
*/
package com.xqual.xtrigger.star.sql;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.utils.CSqlUtils;
import com.xqual.xtrigger.CTriggerGlobals;
import com.xqual.xtrigger.star.IStarConstants;


/**
 * The <code>CSqlConnector</code> class defines the connector to send/receive some SQL requests/responses
 * @author egavaldo
 */
public class CStarSqlDeleteEngine implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+


    // simulate pointer on functions
    private final static short FUNCTION_DELETE   = 1;

    
    // disable all the current links to one specific object
    public final static String DELETE_LATEST				=	"DELETE FROM #table_name#_data_tag " +
																"WHERE data_id IN ( " +
																"    SELECT data_id " +
																"    FROM #table_name#_data " +
																"    WHERE #reference_field_name# = #reference_field_value# " +
																") " +
																"AND tag_id = " + IStarConstants.TIMETAG_ID_LATEST;


    public final static String DELETE_ROW =	"DELETE FROM #table_name# " +
    										"WHERE #reference_field_name# = '#reference_field_value#'";

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CStarSqlDeleteEngine() {
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+
	
	
    /**
     * delete all LATEST timetags on one specific object
     */
    public int deleteAllLatest(String tableName, String referenceFieldName, String referenceFieldValue) {
        System.out.println("[Delete] deleting timetag LATEST from " + referenceFieldName + "=" + referenceFieldValue + "...");
        return executeSQLQuery(FUNCTION_DELETE,
        		               CSqlUtils.computeStatement(DELETE_LATEST, new String[][]{
        		            		           {"#table_name#", tableName},
                                               {"#reference_field_name#", referenceFieldName},
                                               {"#reference_field_value#", referenceFieldValue}}));
    }
    
    
    /**
     * delete a list of assigned to
     */
    public int deleteDefectAssignedTo(String defectId) {
    	System.out.println("[Delete] deleting all assigned to of defect " + defectId + "...");
        return executeSQLQuery(FUNCTION_DELETE,
						        DELETE_ROW.replaceFirst("#table_name#", "table_def_int_assigned_to")
						                  .replaceFirst("#reference_field_name#", "defect_id")
						                  .replaceFirst("#reference_field_value#", defectId));
    }

    
	/**
	 * return the result of a query execution
	 */
	public int executeSQLQuery(short functionId, String query) {
        int result = 0;
		Connection connection = null;
		Statement statement = null;
		try {
			connection = CTriggerGlobals.applicationConnectionFactory.getConnection();
			statement = connection.createStatement();

			switch (functionId) {

				case FUNCTION_DELETE:
	                System.out.println("executing query FUNCTION_DELETE...");
	                result = statement.executeUpdate(query);
					break;

				default:
					break;
			}

			if (statement != null)
				try {
					statement.close();
				} catch (SQLException e) {
					result = -1;
					System.err.println("SQLException: " + e.getMessage());
				}
		} catch (SQLException e) {
            result = -1;
            System.err.println("SQLException: " + e.getMessage());
            System.err.println("Query: " + query);
			if (e instanceof java.sql.SQLIntegrityConstraintViolationException) { // probably the item being edited (insert of a 'content') does not exist anymore because deleted by someone else while we were editing it
				System.err.println("the item being edited very likely does not exist anymore");
			} else {
				e.printStackTrace();
			}
        } catch (Exception e) { // probably, the SQL connection has been lost
			result = -1;
			System.err.println("sql connection lost");
			e.printStackTrace();

		} finally {
			if (statement != null)
				try {
					statement.close();
				} catch (SQLException e) {
					result = -1;
					System.err.println("SQLException: " + e.getMessage());
				}
		}
		CTriggerGlobals.applicationConnectionFactory.freeConnection(connection);

		System.err.println("result (id inserted, nb of rows inserted etc.): " + result);
        return result;
	}

	// +==============================================================+
	// | Utilities                                                    |
	// +==============================================================+

	public static String getCondition_Revision_DefInt(String timetagId) {
		return getCondition_Revision("table_def_int", "defect_id", timetagId);
	}
	
	public static String getCondition_Revision(String table, String field, String timetagId) {
		StringBuffer outputBuffer = new StringBuffer();
		outputBuffer.append("AND " + table + "." + field + "=" + table + "_data." + field + " ");
		outputBuffer.append("AND " + table + "_data.data_id=" + table + "_data_tag.data_id ");
		outputBuffer.append("AND " + table + "_data_tag.tag_id=" + timetagId + " ");
		return outputBuffer.toString();
	}
}
