/*
+----------------------------------------------------------------------+
|                    Class: CSqlGetEngine                              |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
+----------------------------------------------------------------------+
*/
package com.xqual.xtrigger.star.sql;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.LinkedHashMap;

import com.xqual.xcommon.IConstantsResults;
import com.xqual.xtrigger.CTriggerGlobals;
import com.xqual.xtrigger.star.CDefectDetails;
import com.xqual.xtrigger.star.IStarConstants;


/**
 * The <code>CSqlConnector</code> class defines the connector to send/receive some SQL requests/responses
 * @author egavaldo
 */
public class CStarSqlGetEngine implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+
	
    // simulate pointer on functions
	public final static short FUNCTION_GET_MAP_STRING_STRING = 1;
    private final static short FUNCTION_GETDEFECT_DETAILS_INTEGRATED = 2;
    private final static short FUNCTION_ARG_1 = 3;

    public final static String GET_ALL_INTEGRATEDDEFECTS_ID_EXTERNALID = "SELECT defect_id, external_id " +
																		 "FROM table_def_int " +
																		 "WHERE !(external_id IS NULL)";

    public final static String GET_ALL_INTEGRATEDDEFECTS_EXTERNALID_STATUS = "SELECT table_def_int.external_id, status " +
																			 "FROM table_def_int, table_def_int_data, table_def_int_data_tag " +
																			 "WHERE !(external_id IS NULL) " +
																			 "#condition_current_revision_defint#";

	public final static String GET_DEFECT_DETAILS_INTEGRATED =	"SELECT table_def_int.defect_id, defect_name, defect_description, priority, severity, status, " +
																"       step_to_reproduce, reproducibility, platform_id, os_id, fix_target_date, fix_target_version, " +
																"       completion_percentage, fix_type, fix_description, fix_patches " +
																"FROM table_def_int, table_def_int_data, table_def_int_data_tag " +
																"WHERE 1=1 " +
																"#condition_current_revision_defint# " +
																"AND table_def_int.defect_id=#defect_id#";
	
    public final static String GET_LATESTREVISION = "SELECT MAX(revision) " +
												    "FROM #table_name# " +
												    "WHERE #reference_field_name#=#reference_field_value# " +
												    "GROUP BY #reference_field_name#"; // required for max()
	
	public final static String GET_DEFECT_REPORTER_NAME =	"SELECT user_name " +
															"FROM table_def_int, table_user " +
															"WHERE defect_id=#defect_id# " +
															"AND table_def_int.reporter = table_user.user_id";
	
	public final static String GET_DEFECT_ASSIGNEE_NAME =	"SELECT user_name " +
															"FROM table_def_int_assigned_to, table_user " +
															"WHERE defect_id=#defect_id# " +
															"AND table_def_int_assigned_to.assigned_to = table_user.user_id";

	public final static String GET_DEFECT_EXTERNALID =	"SELECT external_id " +
														"FROM table_def_int " +
														"WHERE defect_id=#defect_id#";
		
	public final static String GET_USER_ID =	"SELECT user_id " +
												"FROM table_user " +
												"WHERE user_name='#user_name#'";
	
	public final static String GET_USER_NAME =	"SELECT user_name " +
												"FROM table_user " +
												"WHERE user_id='#user_id#'";

	public final static String GET_SUT_NAME =	"SELECT sut_name " +
												"FROM table_sut " +
												"WHERE sut_id='#sut_id#'";
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CStarSqlGetEngine() {
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

    public HashMap<String, String> getIntegratedDefectIdExternalId() {
    	return getLinkedHashMapIntegerDescriptor(FUNCTION_GET_MAP_STRING_STRING, GET_ALL_INTEGRATEDDEFECTS_ID_EXTERNALID.replaceAll("#condition_current_revision_defint#", getCondition_Revision_DefInt("" + IStarConstants.TIMETAG_ID_LATEST)));
    }
    
    public HashMap<String, String> getIntegratedDefectExternalIdStatus() {
    	return getLinkedHashMapIntegerDescriptor(FUNCTION_GET_MAP_STRING_STRING, GET_ALL_INTEGRATEDDEFECTS_EXTERNALID_STATUS.replaceAll("#condition_current_revision_defint#", getCondition_Revision_DefInt("" + IStarConstants.TIMETAG_ID_LATEST)));
    }


	public CDefectDetails getDefectIntegrated_Details(String defectId) {
    	return (CDefectDetails)executeSQLQuery_API(FUNCTION_GETDEFECT_DETAILS_INTEGRATED, GET_DEFECT_DETAILS_INTEGRATED.replaceAll("#condition_current_revision_defint#", getCondition_Revision_DefInt("" + IStarConstants.TIMETAG_ID_LATEST)).replaceFirst("#defect_id#", defectId));
    	
    	/*
        String customFields = "";
        if (CStudioGlobals.isAuthorizedAction(IRightConstants.RIGHT_ID_DEFECT_DETAIL_V)) {
        	String lastContentId = executeSQLQuery(FUNCTION_ARG_1, GET_LASTCONTENTID.replaceAll("#table_name#", "table_def_int")
    		                                                                        .replaceFirst("#reference_field_name#", "defect_id")
    		                                                                        .replaceFirst("#reference_field_id#", defectId)
    		    	                                                                .replaceAll("#current_timetag_id#", timetagId));

	        customFields = executeSQLQuery(FUNCTION_GETOBJECT_CUSTOMFIELDS,
	                                       GET_CUSTOM_FIELD_INTEGER_VALUE.replaceFirst("#source_type#", ""+CTreeNodeData.TYPE_DEFECT).replaceFirst("#table_content_name#", "table_def_int_data").replaceFirst("#source_id#", lastContentId),
										   GET_CUSTOM_FIELD_STRING_VALUE.replaceFirst("#source_type#", ""+CTreeNodeData.TYPE_DEFECT).replaceFirst("#table_content_name#", "table_def_int_data").replaceFirst("#source_id#", lastContentId),
										   GET_CUSTOM_FIELD_BOOLEAN_VALUE.replaceFirst("#source_type#", ""+CTreeNodeData.TYPE_DEFECT).replaceFirst("#table_content_name#", "table_def_int_data").replaceFirst("#source_id#", lastContentId),
										   GET_CUSTOM_FIELD_HTML_VALUE.replaceFirst("#source_type#", ""+CTreeNodeData.TYPE_DEFECT).replaceFirst("#table_content_name#", "table_def_int_data").replaceFirst("#source_id#", lastContentId),
										   GET_CUSTOM_FIELD_STRINGCHOICE_VALUE.replaceFirst("#source_type#", ""+CTreeNodeData.TYPE_DEFECT).replaceFirst("#table_content_name#", "table_def_int_data").replaceFirst("#source_id#", lastContentId),
										   "", "", false);
        }
        
        // create the global document
        String aggregatedXmlContent = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?><form>" + standardFields + customFields + "</form>";
		if (!CStudioGlobals.isAuthorizedAction(IRightConstants.RIGHT_ID_DEFECT_DETAIL_E)) {
			aggregatedXmlContent = CConversionUtils.formToReadOnlyForm(aggregatedXmlContent);
		}
		*/
    }
	
	public String getDefect_LatestRevision(String defectId) {
    	return (String)executeSQLQuery_API(FUNCTION_ARG_1, GET_LATESTREVISION.replaceFirst("#table_name#", "table_def_int_data")
																			 .replaceAll("#reference_field_name#", "defect_id")
																			 .replaceFirst("#reference_field_value#", defectId));
	}

    public String getDefectIntegrated_ReporterName(String defectId) {
        return (String)executeSQLQuery_API(FUNCTION_ARG_1, GET_DEFECT_REPORTER_NAME.replaceFirst("#defect_id#", defectId));
    }
    public String getDefectIntegrated_AssigneeName(String defectId) {
        return (String)executeSQLQuery_API(FUNCTION_ARG_1, GET_DEFECT_ASSIGNEE_NAME.replaceFirst("#defect_id#", defectId));
    }
    public String getUserIdFromUserName(String userName) {
        return (String)executeSQLQuery_API(FUNCTION_ARG_1, GET_USER_ID.replaceFirst("#user_name#", userName));
    }
    public String getUserNameFromUserId(String userId) {
        return (String)executeSQLQuery_API(FUNCTION_ARG_1, GET_USER_NAME.replaceFirst("#user_id#", userId));
    }
    public String getSutNameFromSutId(String sutId) {
        return (String)executeSQLQuery_API(FUNCTION_ARG_1, GET_SUT_NAME.replaceFirst("#sut_id#", sutId));
    }

    public String getDefectIntegrated_ExternalId(String defectId) {
        return (String)executeSQLQuery_API(FUNCTION_ARG_1, GET_DEFECT_EXTERNALID.replaceFirst("#defect_id#", defectId));
    }


	/**
	 * return the result (XML) of a query execution
	 */
	public Object executeSQLQuery_API(short functionId, String query) {
		ResultSet resultSet = null;
		Connection connection = null;
		Statement statement = null;
		Object output = null;
		
		try {
			connection = CTriggerGlobals.applicationConnectionFactory.getConnection();
			statement = connection.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY); // options necessary for isFirst() etc.

			switch (functionId) {

	            case FUNCTION_ARG_1 :
	            	System.err.println("executing query FUNCTION_ARG_1...");
					resultSet = statement.executeQuery(query);
					if (resultSet.next()) { // don't use while here !!! this is important to simulate the LIMIT 1 that Oracle does not support
						String arg1 = resultSet.getString(1);
						if (!resultSet.wasNull())
							output = arg1;
					}
					break;


                case FUNCTION_GETDEFECT_DETAILS_INTEGRATED :
                	System.err.println("executing query FUNCTION_GETDEFECT_DETAILS_INTEGRATED...");
                    resultSet = statement.executeQuery(query);
                    if (resultSet.next()) {
                    	String name              = resultSet.getString(2);
                    	String description       = resultSet.getString(3);
                    	String stepsToReproduce  = resultSet.getString(7);
                    	short reproducibility    = (short)resultSet.getInt(8);
                    	int platform             = resultSet.getInt(9);
                    	int os                   = resultSet.getInt(10);
                    	short status             = resultSet.getShort(6);
                    	short severity           = resultSet.getShort(5);
                    	short priority           = resultSet.getShort(4);
                    	String fixTargetDate     = resultSet.getString(11);
                    	String fixTargetVersion  = resultSet.getString(12);
                    	int completion           = resultSet.getInt(13);
                    	short fixType            = (short)resultSet.getInt(14);
                    	String fixDescription    = resultSet.getString(15);
                    	String fixPatches        = resultSet.getString(16);
                     	
                    	output = new CDefectDetails(name, description, stepsToReproduce, reproducibility, platform, os, status, severity, priority, fixTargetDate, fixTargetVersion, completion, fixType, fixDescription, fixPatches);
                   }
                    break;

				default:
					System.err.println("SQL Function Id " + functionId + " not found !");
					break;
			}

            if (resultSet != null)
                try {
                    resultSet.close();
                } catch (SQLException e) {
                	System.err.println("SQLException: " + e.getMessage());
                }

            if (statement != null)
                try {
                    statement.close();
                } catch (SQLException e) {
                	System.err.println("SQLException: " + e.getMessage());
                }
		} catch (SQLException e) {
			System.err.println("SQLException: " + e.getMessage());
			System.err.println("Query: " + query);
			e.printStackTrace();
		} catch (Exception e) {
			System.err.println("sql connection lost");
			e.printStackTrace();
		} finally {
            if (resultSet != null)
				try {
					resultSet.close();
				} catch (SQLException e) {
					System.err.println("SQLException: " + e.getMessage());
				}
			if (statement != null)
				try {
					statement.close();
				} catch (SQLException e) {
					System.err.println("SQLException: " + e.getMessage());
				}
		}

		CTriggerGlobals.applicationConnectionFactory.freeConnection(connection);
		return output;
	}


	
    
	/**
	 * Get all the categories HashMap
	 *
	 * Key   = categoryId
	 * Value = CDescriptor:{???}
	 */
	public HashMap<String, String> getLinkedHashMapIntegerDescriptor(short functionId, String query) {
		ResultSet resultSet = null;
		Connection connection = null;
		Statement statement = null;
		LinkedHashMap<String, String> output = new LinkedHashMap<String, String>();
		
		try {
			connection = CTriggerGlobals.applicationConnectionFactory.getConnection();
			statement = connection.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY); // options necessary for isFirst() etc.
            switch (functionId) {
	                
				case FUNCTION_GET_MAP_STRING_STRING:
	                System.out.println("   > executing FUNCTION_GET_MAP_STRING_STRING...");
	                resultSet = statement.executeQuery(query);
	                while (resultSet.next()) {
	            		output.put(resultSet.getString(1), resultSet.getString(2));
	                }
	                break;

				default:
                    System.err.println("SQL Function Id " + functionId + " not found !");
					break;

            }

	        if (resultSet != null)
	            try {
	                resultSet.close();
	            } catch (SQLException e) {
	                System.err.println("SQLException: " + e.getMessage());
	            }

	        if (statement != null)
	            try {
	                statement.close();
	            } catch (SQLException e) {
	                System.err.println("SQLException: " + e.getMessage());
	            }
		} catch (SQLException e) {
			System.err.println("SQLException: " + e.getMessage());
			System.err.println("Query: " + query);
			e.printStackTrace();

        } catch (Exception e) { // probably, the SQL connection has been lost
            System.err.println("sql connection lost");
			e.printStackTrace();

		} finally {
            if (resultSet != null)
				try {
					resultSet.close();
				} catch (SQLException e) {
					System.err.println("SQLException: " + e.getMessage());

				}
			if (statement != null)
				try {
					statement.close();
				} catch (SQLException e) {
					System.err.println("SQLException: " + e.getMessage());

				}
		}

		CTriggerGlobals.applicationConnectionFactory.freeConnection(connection);

		return output;
	}

	// +==============================================================+
	// | Utilities                                                    |
	// +==============================================================+

	public static String getCondition_Revision_DefInt(String timetagId) {
		return getCondition_Revision("table_def_int", "defect_id", timetagId);
	}
	
	public static String getCondition_Revision(String table, String field, String timetagId) {
		StringBuffer outputBuffer = new StringBuffer();
		outputBuffer.append("AND " + table + "." + field + "=" + table + "_data." + field + " ");
		outputBuffer.append("AND " + table + "_data.data_id=" + table + "_data_tag.data_id ");
		outputBuffer.append("AND " + table + "_data_tag.tag_id=" + timetagId + " ");
		return outputBuffer.toString();
	}
}
