/*
+----------------------------------------------------------------------+
|                    Class: CSqlUpdateEngine                           |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
+----------------------------------------------------------------------+
*/
package com.xqual.xtrigger.star.sql;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Calendar;

import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CSqlUtils;
import com.xqual.xcommon.utils.CTextUtils;
import com.xqual.xtrigger.CTriggerGlobals;
import com.xqual.xtrigger.star.CDefectDetails;
import com.xqual.xtrigger.star.CStarGlobals;
import com.xqual.xtrigger.star.CStarUtils;
import com.xqual.xtrigger.star.IStarConstants;


/**
 * The <code>CSqlConnector</code> class defines the connector to send/receive some SQL requests/responses
 * @author egavaldo
 */
public class CStarSqlUpdateEngine implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

    // simulate pointer on functions
    private final static short FUNCTION_UPDATE   = 1;

    
	
    public final static String UPDATE_ONE_FIELD  =   "UPDATE #table_name# " +
                                                     "SET #field_name# = #field_value# " +
                                                     "WHERE #reference_field_name# = #reference_field_value#";
	

    
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CStarSqlUpdateEngine() {
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	public void setIntegratedDefectStatus(String defectId, String status) {
	    System.out.println("[Update] setIntegratedDefectStatus(" + defectId + "," + status);

	    // get the details now to know in advance everything we need to create a new revision
	    CDefectDetails defectDetails = CStarGlobals.getEngine.getDefectIntegrated_Details(defectId);
	    System.out.println("Defect details:\n" + defectDetails);
	
	    if (!status.equals("" + defectDetails.status)) {
	    	int oldRevision = 1;
			try {
				oldRevision = Integer.parseInt(CStarGlobals.getEngine.getDefect_LatestRevision(defectId));
			    System.out.println("Old revision:\n" + oldRevision);
			} catch (NumberFormatException e) {
			}

	        int defectContentId = CStarGlobals.createEngine.insertDefectIntegratedContent(defectId,
																						  defectDetails.description, ""+defectDetails.priority, ""+defectDetails.severity, ""+status,
																						  defectDetails.stepsToReproduce, ""+defectDetails.reproducibility,
																						  ""+defectDetails.platform, ""+defectDetails.os,
																						  defectDetails.fixTargetDate, defectDetails.fixTargetVersion, ""+defectDetails.completion,
																						  ""+defectDetails.fixType, defectDetails.fixDescription, defectDetails.fixPatches,
																						  ""+(oldRevision+1));

        	// move the LATEST timetag to this new version
            int insertedDefectContentTimetagId = CStarGlobals.createEngine.updateDefectContentLatestTimetag(defectId, ""+defectContentId);
            if (insertedDefectContentTimetagId <= 0) {
	        	System.err.println("Error affecting latest timetag");
            }
            
    		// create the change list
    		Calendar sooner = CCalendarUtils.getTimezonedInstance();
    		Calendar now = CCalendarUtils.getTimezonedInstance("GMT");
            long millis = now.getTimeInMillis(); // insert the details change as the first
            sooner.setTimeInMillis(millis-1000);

        	CStarGlobals.createEngine.insertChange(""+IStarConstants.TYPE_DEFECT, ""+defectId, CCalendarUtils.calendarToString(now), ""+IStarConstants.ADMIN,
												   ""+IStarConstants.ACTION_MODIFY_STATUS,
												   CStarUtils.statusToString(defectDetails.status) + " --> " + CStarUtils.statusToString(Short.parseShort(status)));
	    }
	}
	

	public void setDefectFixDescription(String defectId, String fixDescription) {
	    System.out.println("[Update] setDefectFixDescription(" + defectId + "," + fixDescription);

	    // get the details now to know in advance everything we need to create a new revision
	    CDefectDetails defectDetails = CStarGlobals.getEngine.getDefectIntegrated_Details(defectId);
	    System.out.println("Defect details:\n" + defectDetails);
	
	    if (!fixDescription.equals("" + defectDetails.fixDescription)) {
	    	int oldRevision = 1;
			try {
				oldRevision = Integer.parseInt(CStarGlobals.getEngine.getDefect_LatestRevision(defectId));
			    System.out.println("Old revision:\n" + oldRevision);
			} catch (NumberFormatException e) {
			}

	        int defectContentId = CStarGlobals.createEngine.insertDefectIntegratedContent(defectId,
																						  defectDetails.description, ""+defectDetails.priority, ""+defectDetails.severity, ""+defectDetails.status,
																						  defectDetails.stepsToReproduce, ""+defectDetails.reproducibility,
																						  ""+defectDetails.platform, ""+defectDetails.os,
																						  defectDetails.fixTargetDate, defectDetails.fixTargetVersion, "100",
																						  ""+defectDetails.fixType, fixDescription, defectDetails.fixPatches,
																						  ""+(oldRevision+1));

        	// move the LATEST timetag to this new version
            int insertedDefectContentTimetagId = CStarGlobals.createEngine.updateDefectContentLatestTimetag(defectId, ""+defectContentId);
            if (insertedDefectContentTimetagId <= 0) {
	        	System.err.println("Error affecting latest timetag");
            }
            
    		// create the change list
    		Calendar sooner = CCalendarUtils.getTimezonedInstance();
    		Calendar now = CCalendarUtils.getTimezonedInstance("GMT");
            long millis = now.getTimeInMillis(); // insert the details change as the first
            sooner.setTimeInMillis(millis-1000);

        	CStarGlobals.createEngine.insertChange(""+IStarConstants.TYPE_DEFECT, ""+defectId, CCalendarUtils.calendarToString(now), ""+IStarConstants.ADMIN,
												   ""+IStarConstants.ACTION_MODIFY_FIXDESCRIPTION,
												   defectDetails.fixDescription + " --> " + fixDescription);
	    }
	}

	
	public void setIntegratedDefectAssignee(String defectId, String oldAssigneeName, String newAssigneeName) {
		String newAssigneeId = CStarGlobals.getEngine.getUserIdFromUserName(newAssigneeName);
		if (newAssigneeId == null) {
			System.err.println("Error: assignee not found");
		}
		
		int result = CStarGlobals.createEngine.affectDefectAssignedTo(newAssigneeId, defectId);
        if (result < 0) {
        	System.err.println("Error affecting assignee");
        }

        if (result != -1) {
            // changes
            Calendar now = CCalendarUtils.getTimezonedInstance("GMT");
        	CStarGlobals.createEngine.insertChange(""+IStarConstants.TYPE_DEFECT, ""+defectId, CCalendarUtils.calendarToString(now), ""+IStarConstants.ADMIN,
         			                               ""+IStarConstants.ACTION_MODIFY_ASSIGNEDTO,
         			                               oldAssigneeName + " --> " + newAssigneeName);
        }
	}
	

	public void setIntegratedDefectExternalId(String defectId, String externalId) {
	    System.out.println("[Update] setIntegratedDefectExternalId(" + defectId + "," + externalId + ")");

        executeSQLQuery(FUNCTION_UPDATE, CSqlUtils.computeStatement(UPDATE_ONE_FIELD, new String[][]{
							   {"#table_name#", "table_def_int"},
							   {"#reference_field_name#", "defect_id"},
							   {"#reference_field_value#", defectId},
							   {"#field_name#", "external_id"},
							   {"#field_value#", "'" + CTextUtils.escapeStringForDatabase(externalId) + "'"}}));
	}
	

	public void setIntegratedDefectName(String defectId, String name) {
	    System.out.println("[Update] setIntegratedDefectName(" + defectId + "," + name);

        executeSQLQuery(FUNCTION_UPDATE, CSqlUtils.computeStatement(UPDATE_ONE_FIELD, new String[][]{
							   {"#table_name#", "table_def_int"},
							   {"#reference_field_name#", "defect_id"},
							   {"#reference_field_value#", defectId},
							   {"#field_name#", "defect_name"},
							   {"#field_value#", "'" + CTextUtils.escapeStringForDatabase(name) + "'"}}));
	}
	


	/**
	 * return the result of a query execution
	 */
	public int executeSQLQuery(short functionId, String query) {
        int result = 0;
		Connection connection = null;
		Statement statement = null;
		try {
			connection = CTriggerGlobals.applicationConnectionFactory.getConnection();
			statement = connection.createStatement();

			switch (functionId) {

				case FUNCTION_UPDATE:
					System.out.println("   > executing query FUNCTION_UPDATE...");
                    
                    // the int value returned shows how much rows are affected by the sql statement.
                    // for example how much rows are deleted / updated / inserted.
                    // it returns 0 when the statement does not affect any row, for example in DDL execution it may return 0.
                    
                    // it can be useful to know when an update did not change anything (i.e. when update a testcase execution message to understand if a result has already been submitted)
                    // but most of the time just test if the result = -1 for errors
                    result = statement.executeUpdate(query);
					break;

				default:
					break;
			}

			if (statement != null)
				try {
					statement.close();
				} catch (SQLException e) {
					result = -1;
					System.err.println("SQLException: " + e.getMessage());
				}
		} catch (SQLException e) {
            result = -1;
            System.err.println("SQLException: " + e.getMessage());
            System.err.println("Query: " + query);
			if (e instanceof java.sql.SQLIntegrityConstraintViolationException) { // probably the item being edited (insert of a 'content') does not exist anymore because deleted by someone else while we were editing it
				System.err.println("the item being edited very likely does not exist anymore");
			} else {
				e.printStackTrace();
			}
        } catch (Exception e) { // probably, the SQL connection has been lost
			result = -1;
			System.err.println("sql connection lost");
			e.printStackTrace();

		} finally {
			if (statement != null)
				try {
					statement.close();
				} catch (SQLException e) {
					result = -1;
					System.err.println("SQLException: " + e.getMessage());
				}
		}
		CTriggerGlobals.applicationConnectionFactory.freeConnection(connection);

		System.err.println("result (id inserted, nb of rows inserted etc.): " + result);
        return result;
	}

	// +==============================================================+
	// | Utilities                                                    |
	// +==============================================================+

	public static String getCondition_Revision_DefInt(String timetagId) {
		return getCondition_Revision("table_def_int", "defect_id", timetagId);
	}
	
	public static String getCondition_Revision(String table, String field, String timetagId) {
		StringBuffer outputBuffer = new StringBuffer();
		outputBuffer.append("AND " + table + "." + field + "=" + table + "_data." + field + " ");
		outputBuffer.append("AND " + table + "_data.data_id=" + table + "_data_tag.data_id ");
		outputBuffer.append("AND " + table + "_data_tag.tag_id=" + timetagId + " ");
		return outputBuffer.toString();
	}
}
